@@ -28,15 +28,7 @@
 #include <gconf/gconf-client.h>
 
 #include "gpds-touchpad-definitions.h"
-
-
-static const gchar *touchpad_device_names[] =
-{
-    "SynPS/2 Synaptics TouchPad",
-    "AlpsPS/2 ALPS GlidePoint"
-};
-
-static const gint n_touchpad_device_names = G_N_ELEMENTS(touchpad_device_names);
+#include "gpds-touchpad-xinput.h"
 
 #define GPDS_TYPE_TOUCHPAD_UI            (gpds_touchpad_ui_get_type())
 #define GPDS_TOUCHPAD_UI(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GPDS_TYPE_TOUCHPAD_UI, GpdsTouchpadUI))
@@ -113,9 +105,6 @@ gpds_touchpad_ui_init (GpdsTouchpadUI *ui)
 G_MODULE_EXPORT void
 GPDS_MODULE_IMPL_INIT (GTypeModule *type_module)
 {
-    if (GPDS_TYPE_TOUCHPAD_UI)
-        return;
-
     gpds_touchpad_ui_register_type(type_module);
 }
 
@@ -164,27 +153,28 @@ show_error (GError *error)
 static void
 set_widget_sensitivity (GtkBuilder *builder,
                         const gchar *widget_id, 
-                        GtkToggleButton *button)
+                        gboolean sensitivity)
 {
     GObject *object;
 
     object = gtk_builder_get_object(builder, widget_id);
-    gtk_widget_set_sensitive(GTK_WIDGET(object),
-                             gtk_toggle_button_get_active(button));
+    gtk_widget_set_sensitive(GTK_WIDGET(object), sensitivity);
 }
 
 static void
-set_toggle_property (GpdsXInput *xinput, GtkToggleButton *button, const gchar *property_name)
+set_toggle_property (GpdsXInput *xinput, GtkToggleButton *button, GpdsTouchpadProperty property)
 {
     GError *error = NULL;
-    gboolean active;
+    gint properties[1];
 
-    active = gtk_toggle_button_get_active(button);
-    if (!gpds_xinput_set_property(xinput,
-                                  property_name,
-                                  &error,
-                                  active ? 1 : 0,
-                                  NULL)) {
+    properties[0] = gtk_toggle_button_get_active(button) ? 1 : 0;
+
+    if (!gpds_xinput_set_int_properties(xinput,
+                                        gpds_touchpad_xinput_get_name(property),
+                                        gpds_touchpad_xinput_get_format_type(property),
+                                        &error,
+                                        properties,
+                                        1)) {
         if (error) {
             show_error(error);
             g_error_free(error);
@@ -196,24 +186,25 @@ static void
 set_edge_scroll_toggle_property (GpdsXInput *xinput, GtkBuilder *builder)
 {
     GError *error = NULL;
-    gboolean vertical_scrolling_active;
-    gboolean horizontal_scrolling_active;
     GObject *object;
+    gint properties[3];
 
-    object = gtk_builder_get_object(builder, "vertical_scroll_check");
-    vertical_scrolling_active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(object));
-    set_widget_sensitivity(builder, "vertical_scroll_box", GTK_TOGGLE_BUTTON(object));
+    object = gtk_builder_get_object(builder, "vertical_scrolling");
+    properties[0] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(object)) ? 1 :0;
+    set_widget_sensitivity(builder, "vertical_scrolling_box", properties[0]);
 
-    object = gtk_builder_get_object(builder, "horizontal_scroll_check");
-    horizontal_scrolling_active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(object));
-    set_widget_sensitivity(builder, "horizontal_scroll_box", GTK_TOGGLE_BUTTON(object));
+    object = gtk_builder_get_object(builder, "horizontal_scrolling");
+    properties[1] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(object)) ? 1 :0;
+    set_widget_sensitivity(builder, "horizontal_scrolling_box", properties[1]);
 
-    if (!gpds_xinput_set_property(xinput,
-                                  GPDS_TOUCHPAD_EDGE_SCROLLING,
-                                  &error,
-                                  vertical_scrolling_active ? 1 : 0,
-                                  horizontal_scrolling_active ? 1 : 0,
-                                  NULL)) {
+    properties[2] = 0;
+
+    if (!gpds_xinput_set_int_properties(xinput,
+                                        gpds_touchpad_xinput_get_name(GPDS_TOUCHPAD_EDGE_SCROLLING),
+                                        gpds_touchpad_xinput_get_format_type(GPDS_TOUCHPAD_EDGE_SCROLLING),
+                                        &error,
+                                        properties,
+                                        3)) {
         if (error) {
             show_error(error);
             g_error_free(error);
@@ -222,17 +213,18 @@ set_edge_scroll_toggle_property (GpdsXInput *xinput, GtkBuilder *builder)
 }
 
 static void
-set_range_property (GpdsXInput *xinput, GtkRange *range, const gchar *property_name)
+set_range_property (GpdsXInput *xinput, GtkRange *range, GpdsTouchpadProperty property)
 {
     GError *error = NULL;
-    gdouble value;
+    gint properties[1];
 
-    value = gtk_range_get_value(range);
-    if (!gpds_xinput_set_property(xinput,
-                                  property_name,
-                                  &error,
-                                  (gint)value,
-                                  NULL)) {
+    properties[0] = (gint)gtk_range_get_value(range);
+    if (!gpds_xinput_set_int_properties(xinput,
+                                        gpds_touchpad_xinput_get_name(property),
+                                        gpds_touchpad_xinput_get_format_type(property),
+                                        &error,
+                                        properties,
+                                        1)) {
         if (error) {
             show_error(error);
             g_error_free(error);
@@ -244,22 +236,42 @@ static void
 set_scrolling_distance_range_property (GpdsXInput *xinput, GtkBuilder *builder)
 {
     GError *error = NULL;
-    gdouble vertical_scrolling_distance;
-    gdouble horizontal_scrolling_distance;
     GObject *object;
+    gint properties[2];
 
-    object = gtk_builder_get_object(builder, "horizontal_scroll_scale");
-    vertical_scrolling_distance = gtk_range_get_value(GTK_RANGE(object));
+    object = gtk_builder_get_object(builder, "horizontal_scrolling_scale");
+    properties[0] = (gint)gtk_range_get_value(GTK_RANGE(object));
 
-    object = gtk_builder_get_object(builder, "vertical_scroll_scale");
-    horizontal_scrolling_distance = gtk_range_get_value(GTK_RANGE(object));
+    object = gtk_builder_get_object(builder, "vertical_scrolling_scale");
+    properties[1] = (gint)gtk_range_get_value(GTK_RANGE(object));
 
-    if (!gpds_xinput_set_property(xinput,
-                                  GPDS_TOUCHPAD_SCROLLING_DISTANCE,
-                                  &error,
-                                  (gint)vertical_scrolling_distance,
-                                  (gint)horizontal_scrolling_distance,
-                                  NULL)) {
+    if (!gpds_xinput_set_int_properties(xinput,
+                                        gpds_touchpad_xinput_get_name(GPDS_TOUCHPAD_SCROLLING_DISTANCE),
+                                        gpds_touchpad_xinput_get_format_type(GPDS_TOUCHPAD_SCROLLING_DISTANCE),
+                                        &error,
+                                        properties,
+                                        2)) {
+        if (error) {
+            show_error(error);
+            g_error_free(error);
+        }
+    }
+}
+
+static void
+set_circular_scrolling_trigger_property (GpdsTouchpadUI *ui, GpdsTouchpadCircularScrollingTrigger trigger)
+{
+    GError *error = NULL;
+    gint properties[1];
+
+    properties[0] = trigger;
+
+    if (!gpds_xinput_set_int_properties(ui->xinput,
+                                        gpds_touchpad_xinput_get_name(GPDS_TOUCHPAD_CIRCULAR_SCROLLING_TRIGGER),
+                                        gpds_touchpad_xinput_get_format_type(GPDS_TOUCHPAD_CIRCULAR_SCROLLING_TRIGGER),
+                                        &error,
+                                        properties,
+                                        1)) {
         if (error) {
             show_error(error);
             g_error_free(error);
@@ -271,11 +283,8 @@ static void
 cb_tapping_time_scale_value_changed (GtkRange *range, gpointer user_data)
 {
     GpdsTouchpadUI *ui = GPDS_TOUCHPAD_UI(user_data);
-    GtkBuilder *builder;
     gdouble time;
 
-    builder = gpds_ui_get_builder(GPDS_UI(user_data));
-
     set_range_property(ui->xinput, range, GPDS_TOUCHPAD_TAP_TIME);
 
     time = gtk_range_get_value(range);
@@ -286,18 +295,15 @@ static void
 cb_faster_tapping_check_toggled (GtkToggleButton *button, gpointer user_data)
 {
     GpdsTouchpadUI *ui = GPDS_TOUCHPAD_UI(user_data);
-    GtkBuilder *builder;
     gboolean check;
 
-    builder = gpds_ui_get_builder(GPDS_UI(user_data));
-
     set_toggle_property(ui->xinput, button, GPDS_TOUCHPAD_TAP_FAST_TAP);
     check = gtk_toggle_button_get_active(button);
     gconf_client_set_bool(ui->gconf, GPDS_TOUCHPAD_TAP_FAST_TAP_KEY, check, NULL);
 }
 
 static void
-cb_circular_scroll_check_toggled (GtkToggleButton *button, gpointer user_data)
+cb_circular_scrolling_toggled (GtkToggleButton *button, gpointer user_data)
 {
     GpdsTouchpadUI *ui = GPDS_TOUCHPAD_UI(user_data);
     GtkBuilder *builder;
@@ -306,14 +312,14 @@ cb_circular_scroll_check_toggled (GtkToggleButton *button, gpointer user_data)
     builder = gpds_ui_get_builder(GPDS_UI(user_data));
 
     set_toggle_property(ui->xinput, button, GPDS_TOUCHPAD_CIRCULAR_SCROLLING);
-    set_widget_sensitivity(builder, "circular_scroll_box", button);
 
     check = gtk_toggle_button_get_active(button);
+    set_widget_sensitivity(builder, "circular_scrolling_box", check);
     gconf_client_set_bool(ui->gconf, GPDS_TOUCHPAD_CIRCULAR_SCROLLING_KEY, check, NULL);
 }
 
 static void
-cb_vertical_scroll_check_toggled (GtkToggleButton *button, gpointer user_data)
+cb_vertical_scrolling_toggled (GtkToggleButton *button, gpointer user_data)
 {
     GpdsTouchpadUI *ui = GPDS_TOUCHPAD_UI(user_data);
     GtkBuilder *builder;
@@ -328,7 +334,7 @@ cb_vertical_scroll_check_toggled (GtkToggleButton *button, gpointer user_data)
 }
 
 static void
-cb_horizontal_scroll_check_toggled (GtkToggleButton *button, gpointer user_data)
+cb_horizontal_scrolling_toggled (GtkToggleButton *button, gpointer user_data)
 {
     GpdsTouchpadUI *ui = GPDS_TOUCHPAD_UI(user_data);
     GtkBuilder *builder;
@@ -343,7 +349,7 @@ cb_horizontal_scroll_check_toggled (GtkToggleButton *button, gpointer user_data)
 }
 
 static void
-cb_vertical_scroll_scale_value_changed (GtkRange *range, gpointer user_data)
+cb_vertical_scrolling_scale_value_changed (GtkRange *range, gpointer user_data)
 {
     GpdsTouchpadUI *ui = GPDS_TOUCHPAD_UI(user_data);
     GtkBuilder *builder;
@@ -358,7 +364,7 @@ cb_vertical_scroll_scale_value_changed (GtkRange *range, gpointer user_data)
 }
 
 static void
-cb_horizontal_scroll_scale_value_changed (GtkRange *range, gpointer user_data)
+cb_horizontal_scrolling_scale_value_changed (GtkRange *range, gpointer user_data)
 {
     GpdsTouchpadUI *ui = GPDS_TOUCHPAD_UI(user_data);
     GtkBuilder *builder;
@@ -372,6 +378,147 @@ cb_horizontal_scroll_scale_value_changed (GtkRange *range, gpointer user_data)
     gconf_client_set_int(ui->gconf, GPDS_TOUCHPAD_VERTICAL_SCROLLING_DISTANCE_KEY, (gint)distance, NULL);
 }
 
+static void
+set_circular_scrolling_trigger_button_state (GpdsTouchpadUI *ui, 
+                                             GpdsTouchpadCircularScrollingTrigger trigger)
+{
+    GtkToggleButton *button;
+    GtkBuilder *builder;
+    gboolean active = FALSE;
+
+    builder = gpds_ui_get_builder(GPDS_UI(ui));
+
+#define SET_TOGGLE_BUTTON_STATE(name, state)                           \
+    button = GTK_TOGGLE_BUTTON(gtk_builder_get_object(builder, name)); \
+    gtk_toggle_button_set_active(button, state);
+
+    if (trigger == GPDS_TOUCHPAD_CIRCULAR_SCROLLING_TRIGGER_ANY)
+        active = TRUE;
+
+    SET_TOGGLE_BUTTON_STATE("trigger_top_toggle", active);
+    SET_TOGGLE_BUTTON_STATE("trigger_top_right_toggle", active);
+    SET_TOGGLE_BUTTON_STATE("trigger_right_toggle", active);
+    SET_TOGGLE_BUTTON_STATE("trigger_right_bottom_toggle", active);
+    SET_TOGGLE_BUTTON_STATE("trigger_bottom_toggle", active);
+    SET_TOGGLE_BUTTON_STATE("trigger_bottom_left_toggle", active);
+    SET_TOGGLE_BUTTON_STATE("trigger_left_toggle", active);
+    SET_TOGGLE_BUTTON_STATE("trigger_left_top_toggle", active);
+
+    switch (trigger) {
+    case GPDS_TOUCHPAD_CIRCULAR_SCROLLING_TRIGGER_TOP:
+        SET_TOGGLE_BUTTON_STATE("trigger_top_toggle", TRUE);
+        break;
+    case GPDS_TOUCHPAD_CIRCULAR_SCROLLING_TRIGGER_TOP_RIGHT:
+        SET_TOGGLE_BUTTON_STATE("trigger_top_right_toggle", TRUE);
+        break;
+    case GPDS_TOUCHPAD_CIRCULAR_SCROLLING_TRIGGER_RIGHT:
+        SET_TOGGLE_BUTTON_STATE("trigger_right_toggle", TRUE);
+        break;
+    case GPDS_TOUCHPAD_CIRCULAR_SCROLLING_TRIGGER_RIGHT_BOTTOM:
+        SET_TOGGLE_BUTTON_STATE("trigger_right_bottom_toggle", TRUE);
+        break;
+    case GPDS_TOUCHPAD_CIRCULAR_SCROLLING_TRIGGER_BOTTOM:
+        SET_TOGGLE_BUTTON_STATE("trigger_bottom_toggle", TRUE);
+        break;
+    case GPDS_TOUCHPAD_CIRCULAR_SCROLLING_TRIGGER_BOTTOM_LEFT:
+        SET_TOGGLE_BUTTON_STATE("trigger_bottom_left_toggle", TRUE);
+        break;
+    case GPDS_TOUCHPAD_CIRCULAR_SCROLLING_TRIGGER_LEFT:
+        SET_TOGGLE_BUTTON_STATE("trigger_left_toggle", TRUE);
+        break;
+    case GPDS_TOUCHPAD_CIRCULAR_SCROLLING_TRIGGER_LEFT_TOP:
+        SET_TOGGLE_BUTTON_STATE("trigger_left_top_toggle", TRUE);
+        break;
+    case GPDS_TOUCHPAD_CIRCULAR_SCROLLING_TRIGGER_ANY:
+    default:
+        break;
+    }
+#undef SET_TOGGLE_BUTTON_STATE
+}
+
+#define DEFINE_CIRCULAR_SCROLLING_TRIGGER_CALLBACK(position, trigger)                                   \
+static void                                                                                             \
+cb_trigger_ ## position ## _toggle_button_press_event (GtkWidget *widget,                               \
+                                                       GdkEventButton *event,                           \
+                                                       gpointer user_data)                              \
+{                                                                                                       \
+    set_circular_scrolling_trigger_property(GPDS_TOUCHPAD_UI(user_data),                                \
+                                            GPDS_TOUCHPAD_CIRCULAR_SCROLLING_TRIGGER_ ## trigger);      \
+    set_circular_scrolling_trigger_button_state(GPDS_TOUCHPAD_UI(user_data),                            \
+                                                GPDS_TOUCHPAD_CIRCULAR_SCROLLING_TRIGGER_ ## trigger);  \
+}
+
+DEFINE_CIRCULAR_SCROLLING_TRIGGER_CALLBACK(top, TOP)
+DEFINE_CIRCULAR_SCROLLING_TRIGGER_CALLBACK(top_right, TOP_RIGHT)
+DEFINE_CIRCULAR_SCROLLING_TRIGGER_CALLBACK(right, RIGHT)
+DEFINE_CIRCULAR_SCROLLING_TRIGGER_CALLBACK(right_bottom, RIGHT_BOTTOM)
+DEFINE_CIRCULAR_SCROLLING_TRIGGER_CALLBACK(bottom, BOTTOM)
+DEFINE_CIRCULAR_SCROLLING_TRIGGER_CALLBACK(bottom_left, BOTTOM_LEFT)
+DEFINE_CIRCULAR_SCROLLING_TRIGGER_CALLBACK(left, LEFT)
+DEFINE_CIRCULAR_SCROLLING_TRIGGER_CALLBACK(left_top, LEFT_TOP)
+DEFINE_CIRCULAR_SCROLLING_TRIGGER_CALLBACK(any, ANY)
+
+#undef DEFINE_CIRCULAR_SCROLLING_TRIGGER_CALLBACK
+
+static void
+set_sensitivity_depends_on_use_type (GpdsTouchpadUI *ui,
+                                     GpdsTouchpadUseType use_type)
+{
+    GtkBuilder *builder;
+
+    builder = gpds_ui_get_builder(GPDS_UI(ui));
+
+    switch (use_type) {
+    case GPDS_TOUCHPAD_USE_TYPE_OFF:
+    case GPDS_TOUCHPAD_USE_TYPE_TAPPING_AND_SCROLLING_OFF:
+        set_widget_sensitivity(builder, "scrolling_vbox", FALSE);
+        set_widget_sensitivity(builder, "tapping_vbox", FALSE);
+        break;
+    case GPDS_TOUCHPAD_USE_TYPE_NORMAL:
+        set_widget_sensitivity(builder, "scrolling_vbox", TRUE);
+        set_widget_sensitivity(builder, "tapping_vbox", TRUE);
+    default:
+        break;
+    }
+}
+
+static void
+set_touchpad_use_type_combo_state (GpdsTouchpadUI *ui, 
+                                   GpdsTouchpadUseType use_type)
+{
+    GtkComboBox *combo;
+    GtkBuilder *builder;
+
+    builder = gpds_ui_get_builder(GPDS_UI(ui));
+
+    combo = GTK_COMBO_BOX(gtk_builder_get_object(builder, "touchpad_use_type"));
+    gtk_combo_box_set_active(combo, (gint)use_type);
+    set_sensitivity_depends_on_use_type(ui, use_type);
+}
+
+static void
+cb_touchpad_use_type_changed (GtkComboBox *combo, gpointer user_data)
+{
+    GpdsTouchpadUI *ui = GPDS_TOUCHPAD_UI(user_data);
+    gint properties[1];
+    GError *error = NULL;
+
+    properties[0] = gtk_combo_box_get_active(combo);
+    if (!gpds_xinput_set_int_properties(ui->xinput,
+                                        gpds_touchpad_xinput_get_name(GPDS_TOUCHPAD_OFF),
+                                        gpds_touchpad_xinput_get_format_type(GPDS_TOUCHPAD_OFF),
+                                        &error,
+                                        properties,
+                                        1)) {
+        if (error) {
+            show_error(error);
+            g_error_free(error);
+        }
+    }
+    gconf_client_set_bool(ui->gconf, GPDS_TOUCHPAD_OFF_KEY, properties[0], NULL);
+    set_sensitivity_depends_on_use_type(ui, properties[0]);
+}
+
 static void
 setup_signals (GpdsUI *ui, GtkBuilder *builder)
 {
@@ -383,27 +530,39 @@ setup_signals (GpdsUI *ui, GtkBuilder *builder)
                      G_CALLBACK(cb_ ## object_name ## _ ## signal_name),\
                      ui)
 
+    CONNECT(touchpad_use_type, changed);
     CONNECT(tapping_time_scale, value_changed);
     CONNECT(faster_tapping_check, toggled);
-    CONNECT(circular_scroll_check, toggled);
-    CONNECT(vertical_scroll_check, toggled);
-    CONNECT(vertical_scroll_scale, value_changed);
-    CONNECT(horizontal_scroll_check, toggled);
-    CONNECT(horizontal_scroll_scale, value_changed);
+    CONNECT(circular_scrolling, toggled);
+    CONNECT(vertical_scrolling, toggled);
+    CONNECT(vertical_scrolling_scale, value_changed);
+    CONNECT(horizontal_scrolling, toggled);
+    CONNECT(horizontal_scrolling_scale, value_changed);
+
+    /* cirlular scrolling trigger */
+    CONNECT(trigger_top_toggle, button_press_event);
+    CONNECT(trigger_top_right_toggle, button_press_event);
+    CONNECT(trigger_right_toggle, button_press_event);
+    CONNECT(trigger_right_bottom_toggle, button_press_event);
+    CONNECT(trigger_bottom_toggle, button_press_event);
+    CONNECT(trigger_bottom_left_toggle, button_press_event);
+    CONNECT(trigger_left_toggle, button_press_event);
+    CONNECT(trigger_left_top_toggle, button_press_event);
+    CONNECT(trigger_any_toggle, button_press_event);
 
 #undef CONNECT
 }
 
 static gboolean
-get_integer_property (GpdsXInput *xinput, const gchar *property_name,
+get_integer_properties (GpdsXInput *xinput, GpdsTouchpadProperty property,
                       gint **values, gulong *n_values)
 {
     GError *error = NULL;
 
-    if (!gpds_xinput_get_property(xinput,
-                                  property_name,
-                                  &error,
-                                  values, n_values)) {
+    if (!gpds_xinput_get_int_properties(xinput,
+                                        gpds_touchpad_xinput_get_name(property),
+                                        &error,
+                                        values, n_values)) {
         if (error) {
             show_error(error);
             g_error_free(error);
@@ -417,7 +576,7 @@ get_integer_property (GpdsXInput *xinput, const gchar *property_name,
 
 static void
 set_integer_property_from_preference (GpdsTouchpadUI *ui,
-                                      const gchar *property_name,
+                                      GpdsTouchpadProperty property,
                                       const gchar *gconf_key_name,
                                       GtkBuilder *builder,
                                       const gchar *object_name)
@@ -428,8 +587,8 @@ set_integer_property_from_preference (GpdsTouchpadUI *ui,
     gint value;
     gboolean dir_exists;
 
-    if (!get_integer_property(ui->xinput, property_name,
-                              &values, &n_values)) {
+    if (!get_integer_properties(ui->xinput, property,
+                                &values, &n_values)) {
         return;
     }
 
@@ -443,9 +602,9 @@ set_integer_property_from_preference (GpdsTouchpadUI *ui,
     g_free(values);
 }
 
-static void
+static gboolean
 set_boolean_property_from_preference (GpdsTouchpadUI *ui,
-                                      const gchar *property_name,
+                                      GpdsTouchpadProperty property,
                                       const gchar *gconf_key_name,
                                       GtkBuilder *builder,
                                       const gchar *object_name)
@@ -455,9 +614,9 @@ set_boolean_property_from_preference (GpdsTouchpadUI *ui,
     gulong n_values;
     gboolean enable, dir_exists;
 
-    if (!get_integer_property(ui->xinput, property_name,
-                              &values, &n_values)) {
-        return;
+    if (!get_integer_properties(ui->xinput, property,
+                                &values, &n_values)) {
+        return FALSE;
     }
 
     dir_exists = gconf_client_dir_exists(ui->gconf, GPDS_TOUCHPAD_GCONF_DIR, NULL);
@@ -468,6 +627,8 @@ set_boolean_property_from_preference (GpdsTouchpadUI *ui,
     object = gtk_builder_get_object(builder, object_name);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(object), enable);
     g_free(values);
+
+    return enable;
 }
 
 static void
@@ -479,8 +640,8 @@ set_edge_scroll_property_from_preference (GpdsTouchpadUI *ui,
     gulong n_values;
     gboolean enable, dir_exists;
 
-    if (!get_integer_property(ui->xinput, GPDS_TOUCHPAD_EDGE_SCROLLING,
-                              &values, &n_values)) {
+    if (!get_integer_properties(ui->xinput, GPDS_TOUCHPAD_EDGE_SCROLLING,
+                                &values, &n_values)) {
         return;
     }
 
@@ -494,15 +655,17 @@ set_edge_scroll_property_from_preference (GpdsTouchpadUI *ui,
         enable = gconf_client_get_bool(ui->gconf, GPDS_TOUCHPAD_VERTICAL_SCROLLING_KEY, NULL);
     else
         enable = (values[0] == 1);
-    object = gtk_builder_get_object(builder, "vertical_scroll_check");
+    object = gtk_builder_get_object(builder, "vertical_scrolling");
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(object), enable);
+    set_widget_sensitivity(builder, "vertical_scrolling_box", enable);
 
     if (dir_exists)
         enable = gconf_client_get_bool(ui->gconf, GPDS_TOUCHPAD_HORIZONTAL_SCROLLING_KEY, NULL);
     else
         enable = (values[0] == 1);
-    object = gtk_builder_get_object(builder, "horizontal_scroll_check");
+    object = gtk_builder_get_object(builder, "horizontal_scrolling");
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(object), enable);
+    set_widget_sensitivity(builder, "horizontal_scrolling_box", enable);
 
     g_free(values);
 }
@@ -517,9 +680,9 @@ set_scroll_distance_property_from_preference (GpdsTouchpadUI *ui,
     gulong n_values;
     gint distance;
 
-    if (!get_integer_property(ui->xinput,
-                              GPDS_TOUCHPAD_SCROLLING_DISTANCE,
-                              &values, &n_values)) {
+    if (!get_integer_properties(ui->xinput,
+                                GPDS_TOUCHPAD_SCROLLING_DISTANCE,
+                                &values, &n_values)) {
         return;
     }
 
@@ -531,7 +694,7 @@ set_scroll_distance_property_from_preference (GpdsTouchpadUI *ui,
     distance = gconf_client_get_int(ui->gconf,
                                     GPDS_TOUCHPAD_VERTICAL_SCROLLING_DISTANCE_KEY,
                                     &error);
-    object = gtk_builder_get_object(builder, "vertical_scroll_scale");
+    object = gtk_builder_get_object(builder, "vertical_scrolling_scale");
     gtk_range_set_value(GTK_RANGE(object), error ? values[0] : distance);
     if (error)
         g_clear_error(&error);
@@ -539,7 +702,7 @@ set_scroll_distance_property_from_preference (GpdsTouchpadUI *ui,
     distance = gconf_client_get_int(ui->gconf,
                                     GPDS_TOUCHPAD_HORIZONTAL_SCROLLING_DISTANCE_KEY,
                                     &error);
-    object = gtk_builder_get_object(builder, "horizontal_scroll_scale");
+    object = gtk_builder_get_object(builder, "horizontal_scrolling_scale");
     gtk_range_set_value(GTK_RANGE(object), error ? values[1] : distance);
     if (error)
         g_clear_error(&error);
@@ -547,6 +710,71 @@ set_scroll_distance_property_from_preference (GpdsTouchpadUI *ui,
     g_free(values);
 }
 
+static void
+set_circular_scrolling_property_from_preference (GpdsTouchpadUI *ui,
+                                                 GtkBuilder *builder)
+{
+    gboolean enable;
+
+    enable = set_boolean_property_from_preference(ui,
+                                                  GPDS_TOUCHPAD_CIRCULAR_SCROLLING,
+                                                  GPDS_TOUCHPAD_CIRCULAR_SCROLLING_KEY,
+                                                  builder,
+                                                  "circular_scrolling");
+
+    set_widget_sensitivity(builder, "circular_scrolling_box", enable);
+}
+
+static void
+set_circular_scrolling_trigger_property_from_preference (GpdsTouchpadUI *ui,
+                                                         GtkBuilder *builder)
+{
+    GError *error = NULL;
+    gint *values;
+    gulong n_values;
+    GpdsTouchpadCircularScrollingTrigger trigger;
+
+    if (!get_integer_properties(ui->xinput,
+                                GPDS_TOUCHPAD_CIRCULAR_SCROLLING_TRIGGER,
+                                &values, &n_values)) {
+        return;
+    }
+
+    trigger = gconf_client_get_int(ui->gconf,
+                                   GPDS_TOUCHPAD_CIRCULAR_SCROLLING_TRIGGER_KEY,
+                                   &error);
+    set_circular_scrolling_trigger_button_state(ui, error ? values[0] : trigger);
+    if (error)
+        g_clear_error(&error);
+
+    g_free(values);
+}
+
+static void
+set_touchpad_use_type_property_from_preference (GpdsTouchpadUI *ui,
+                                                GtkBuilder *builder)
+{
+    GError *error = NULL;
+    gint *values;
+    gulong n_values;
+    GpdsTouchpadUseType type;
+
+    if (!get_integer_properties(ui->xinput,
+                                GPDS_TOUCHPAD_OFF,
+                                &values, &n_values)) {
+        return;
+    }
+
+    type = gconf_client_get_int(ui->gconf,
+                                GPDS_TOUCHPAD_OFF_KEY,
+                                &error);
+    set_touchpad_use_type_combo_state(ui, error ? values[0] : type);
+    if (error)
+        g_clear_error(&error);
+
+    g_free(values);
+}
+
 static void
 setup_current_values (GpdsUI *ui, GtkBuilder *builder)
 {
@@ -562,32 +790,19 @@ setup_current_values (GpdsUI *ui, GtkBuilder *builder)
                                          GPDS_TOUCHPAD_TAP_FAST_TAP_KEY,
                                          builder,
                                          "faster_tapping_check");
-    set_boolean_property_from_preference(touchpad_ui,
-                                         GPDS_TOUCHPAD_CIRCULAR_SCROLLING,
-                                         GPDS_TOUCHPAD_CIRCULAR_SCROLLING_KEY, 
-                                         builder,
-                                         "circular_scroll_check");
+    set_circular_scrolling_property_from_preference(touchpad_ui, builder);
     set_edge_scroll_property_from_preference(touchpad_ui, builder);
     set_scroll_distance_property_from_preference(touchpad_ui, builder);
-}
+    set_circular_scrolling_trigger_property_from_preference(touchpad_ui, builder);
 
-static const gchar *
-find_device_name (void)
-{
-    gint i;
-
-    for (i = 0; i < n_touchpad_device_names; i++) {
-        if (gpds_xinput_exist_device(touchpad_device_names[i]))
-            return touchpad_device_names[i];
-    }
-    return NULL;
+    set_touchpad_use_type_property_from_preference(touchpad_ui, builder);
 }
 
 static gboolean
 is_available (GpdsUI *ui, GError **error)
 {
     const gchar *device_name;
-    device_name = find_device_name();
+    device_name = gpds_touchpad_xinput_find_device_name();
 
     if (!device_name) {
         g_set_error(error,
