@@ -28,6 +28,7 @@
 #include <gconf/gconf-client.h>
 
 #include "gpds-trackpoint-definitions.h"
+#include "gpds-trackpoint-xinput.h"
 
 #define GPDS_TYPE_TRACK_POINT_UI            (gpds_track_point_ui_get_type())
 #define GPDS_TRACK_POINT_UI(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GPDS_TYPE_TRACK_POINT_UI, GpdsTrackPointUI))
@@ -45,6 +46,7 @@ struct _GpdsTrackPointUI
     GpdsXInput *xinput;
     gchar *ui_file_path;
     GConfClient *gconf;
+    gchar *device_name;
 };
 
 struct _GpdsTrackPointUIClass
@@ -98,14 +100,12 @@ gpds_track_point_ui_init (GpdsTrackPointUI *ui)
                                         "trackpoint.ui",
                                         NULL);
     ui->gconf = gconf_client_get_default();
+    ui->device_name = NULL;
 }
 
 G_MODULE_EXPORT void
 GPDS_MODULE_IMPL_INIT (GTypeModule *type_module)
 {
-    if (GPDS_TYPE_TRACK_POINT_UI)
-        return;
-
     gpds_track_point_ui_register_type(type_module);
 }
 
@@ -136,6 +136,7 @@ dispose (GObject *object)
     }
 
     g_free(ui->ui_file_path);
+    g_free(ui->device_name);
 
     if (G_OBJECT_CLASS(gpds_track_point_ui_parent_class)->dispose)
         G_OBJECT_CLASS(gpds_track_point_ui_parent_class)->dispose(object);
@@ -151,17 +152,25 @@ show_error (GError *error)
 }
 
 static void
-set_toggle_property (GpdsXInput *xinput, GtkToggleButton *button, const gchar *property_name)
+set_toggle_property (GpdsXInput *xinput, GtkToggleButton *button, GpdsTrackPointProperty property)
 {
     GError *error = NULL;
     gboolean active;
+    const gchar *property_name;
+    gint format_type;
+    gint properties[1];
 
     active = gtk_toggle_button_get_active(button);
-    gpds_xinput_set_property(xinput,
-                             property_name,
-                             &error,
-                             active ? 1 : 0,
-                             NULL);
+    property_name = gpds_track_point_xinput_get_name(property);
+    format_type = gpds_track_point_xinput_get_format_type(property);
+
+    properties[0] = active ? 1 : 0;
+    gpds_xinput_set_int_properties(xinput,
+                                   property_name,
+                                   format_type,
+                                   &error,
+                                   properties,
+                                   1);
     if (error) {
         show_error(error);
         g_error_free(error);
@@ -169,17 +178,25 @@ set_toggle_property (GpdsXInput *xinput, GtkToggleButton *button, const gchar *p
 }
 
 static void
-set_spin_property (GpdsXInput *xinput, GtkSpinButton *button, const gchar *property_name)
+set_spin_property (GpdsXInput *xinput, GtkSpinButton *button, GpdsTrackPointProperty property)
 {
     GError *error = NULL;
     gdouble value;
+    const gchar *property_name;
+    gint format_type;
+    gint properties[1];
 
     value = gtk_spin_button_get_value(button);
-    gpds_xinput_set_property(xinput,
-                             property_name,
-                             &error,
-                             (gint)value,
-                             NULL);
+    property_name = gpds_track_point_xinput_get_name(property);
+    format_type = gpds_track_point_xinput_get_format_type(property);
+
+    properties[0] = (gint)value;
+    gpds_xinput_set_int_properties(xinput,
+                                   property_name,
+                                   format_type,
+                                   &error,
+                                   properties,
+                                   1);
     if (error) {
         show_error(error);
         g_error_free(error);
@@ -231,28 +248,47 @@ cb_wheel_emulation_toggled (GtkToggleButton *button, gpointer user_data)
 }
 
 static void
-set_toggle_scroll_property (GpdsXInput *xinput, GtkToggleButton *button,
-                            const gchar *property_name,
-                            gint first_value, gint second_value)
+set_scroll_axes_property (GpdsTrackPointUI *ui)
 {
+    GtkBuilder *builder;
+    GtkToggleButton *button;
     GError *error = NULL;
     gboolean active;
+    const gchar *property_name;
+    gint format_type;
+    gint properties[4];
 
+    builder = gpds_ui_get_builder(GPDS_UI(ui));
+
+    button = GTK_TOGGLE_BUTTON(gtk_builder_get_object(builder, "wheel_emulation_vertical"));
     active = gtk_toggle_button_get_active(button);
     if (active) {
-        gpds_xinput_set_property(xinput,
-                                 property_name,
-                                 &error,
-                                 first_value, second_value,
-                                 NULL);
+        properties[0] = 6;
+        properties[1] = 7;
     } else {
-        gpds_xinput_set_property(xinput,
-                                 property_name,
-                                 &error,
-                                 -1, -1,
-                                 NULL);
+        properties[0] = 0;
+        properties[1] = 0;
     }
 
+    button = GTK_TOGGLE_BUTTON(gtk_builder_get_object(builder, "wheel_emulation_horizontal"));
+    active = gtk_toggle_button_get_active(button);
+    if (active) {
+        properties[2] = 4;
+        properties[3] = 5;
+    } else {
+        properties[2] = 0;
+        properties[3] = 0;
+    }
+
+    property_name = gpds_track_point_xinput_get_name(GPDS_TRACK_POINT_WHEEL_EMULATION_AXES);
+    format_type = gpds_track_point_xinput_get_format_type(GPDS_TRACK_POINT_WHEEL_EMULATION_AXES);
+
+    gpds_xinput_set_int_properties(ui->xinput,
+                                   property_name,
+                                   format_type,
+                                   &error,
+                                   properties,
+                                   4);
     if (error) {
         show_error(error);
         g_error_free(error);
@@ -264,7 +300,7 @@ cb_wheel_emulation_vertical_toggled (GtkToggleButton *button, gpointer user_data
 {
     gboolean enable;
     GpdsTrackPointUI *ui = GPDS_TRACK_POINT_UI(user_data);
-    set_toggle_scroll_property(ui->xinput, button, GPDS_TRACK_POINT_WHEEL_EMULATION_Y_AXIS, 4, 5);
+    set_scroll_axes_property(ui);
 
     enable = gtk_toggle_button_get_active(button);
     gconf_client_set_bool(ui->gconf, GPDS_TRACK_POINT_WHEEL_EMULATION_Y_AXIS_KEY, enable, NULL);
@@ -275,7 +311,7 @@ cb_wheel_emulation_horizontal_toggled (GtkToggleButton *button, gpointer user_da
 {
     gboolean enable;
     GpdsTrackPointUI *ui = GPDS_TRACK_POINT_UI(user_data);
-    set_toggle_scroll_property(ui->xinput, button, GPDS_TRACK_POINT_WHEEL_EMULATION_X_AXIS, 6, 7);
+    set_scroll_axes_property(ui);
 
     enable = gtk_toggle_button_get_active(button);
     gconf_client_set_bool(ui->gconf, GPDS_TRACK_POINT_WHEEL_EMULATION_X_AXIS_KEY, enable, NULL);
@@ -337,15 +373,15 @@ setup_signals (GpdsUI *ui, GtkBuilder *builder)
 }
 
 static gboolean
-get_integer_property (GpdsXInput *xinput, const gchar *property_name,
-                      gint **values, gulong *n_values)
+get_integer_properties (GpdsXInput *xinput, const gchar *property_name,
+                        gint **values, gulong *n_values)
 {
     GError *error = NULL;
 
-    if (!gpds_xinput_get_property(xinput,
-                                  property_name,
-                                  &error,
-                                  values, n_values)) {
+    if (!gpds_xinput_get_int_properties(xinput,
+                                        property_name,
+                                        &error,
+                                        values, n_values)) {
         if (error) {
             show_error(error);
             g_error_free(error);
@@ -359,7 +395,7 @@ get_integer_property (GpdsXInput *xinput, const gchar *property_name,
 
 static void
 set_integer_property_from_preference (GpdsTrackPointUI *ui,
-                                      const gchar *property_name,
+                                      GpdsTrackPointProperty property,
                                       const gchar *gconf_key_name,
                                       GtkBuilder *builder,
                                       const gchar *object_name)
@@ -369,9 +405,12 @@ set_integer_property_from_preference (GpdsTrackPointUI *ui,
     gulong n_values;
     gint value;
     gboolean dir_exists;
+    const gchar *property_name;
 
-    if (!get_integer_property(ui->xinput, property_name,
-                              &values, &n_values)) {
+    property_name = gpds_track_point_xinput_get_name(property);
+
+    if (!get_integer_properties(ui->xinput, property_name,
+                                &values, &n_values)) {
         return;
     }
 
@@ -388,7 +427,7 @@ set_integer_property_from_preference (GpdsTrackPointUI *ui,
 
 static void
 set_boolean_property_from_preference (GpdsTrackPointUI *ui,
-                                      const gchar *property_name,
+                                      GpdsTrackPointProperty property,
                                       const gchar *gconf_key_name,
                                       GtkBuilder *builder,
                                       const gchar *object_name)
@@ -397,9 +436,13 @@ set_boolean_property_from_preference (GpdsTrackPointUI *ui,
     gint *values;
     gulong n_values;
     gboolean enable, dir_exists;
+    const gchar *property_name;
+    gchar *box_name;
 
-    if (!get_integer_property(ui->xinput, property_name,
-                              &values, &n_values)) {
+    property_name = gpds_track_point_xinput_get_name(property);
+
+    if (!get_integer_properties(ui->xinput, property_name,
+                                &values, &n_values)) {
         return;
     }
 
@@ -412,33 +455,49 @@ set_boolean_property_from_preference (GpdsTrackPointUI *ui,
     object = gtk_builder_get_object(builder, object_name);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(object), enable);
     g_free(values);
+
+    box_name = g_strconcat(object_name, "_box", NULL);
+    set_widget_sensitivity (builder, box_name, GTK_TOGGLE_BUTTON(object));
+    g_free(box_name);
 }
 
 static void
-set_scroll_property_from_preference (GpdsTrackPointUI *ui,
-                                     const gchar *property_name,
-                                     const gchar *gconf_key_name,
-                                     GtkBuilder *builder,
-                                     const gchar *object_name)
+set_scroll_axes_property_from_preference (GpdsTrackPointUI *ui,
+                                          GtkBuilder *builder)
 {
     GObject *object;
     gint *values;
     gulong n_values;
-    gboolean enable, dir_exists;
+    gboolean horizontal_enable = FALSE, vertical_enable = FALSE, dir_exists;
+    const gchar *property_name;
 
-    if (!get_integer_property(ui->xinput, property_name,
-                              &values, &n_values)) {
+    property_name = gpds_track_point_xinput_get_name(GPDS_TRACK_POINT_WHEEL_EMULATION_AXES);
+
+    if (!get_integer_properties(ui->xinput, property_name,
+                                &values, &n_values)) {
         return;
     }
 
     dir_exists = gconf_client_dir_exists(ui->gconf, GPDS_TRACK_POINT_GCONF_DIR, NULL);
-    if (dir_exists)
-        enable = gconf_client_get_bool(ui->gconf, gconf_key_name, NULL);
-    else
-        enable = (values[0] != 0 && values[1] != 0);
-        
-    object = gtk_builder_get_object(builder, object_name);
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(object), enable);
+    if (dir_exists) {
+        horizontal_enable = gconf_client_get_bool(ui->gconf,
+                                                  GPDS_TRACK_POINT_WHEEL_EMULATION_X_AXIS_KEY,
+                                                  NULL);
+        vertical_enable = gconf_client_get_bool(ui->gconf,
+                                                GPDS_TRACK_POINT_WHEEL_EMULATION_Y_AXIS_KEY,
+                                                NULL);
+    } else {
+        if (n_values >= 2)
+            horizontal_enable = (values[0] != 0 && values[1] != 0);
+        if (n_values >= 4)
+            vertical_enable = (values[2] != 0 && values[3] != 0);
+    }
+
+    object = gtk_builder_get_object(builder, "wheel_emulation_horizontal");
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(object), horizontal_enable);
+    object = gtk_builder_get_object(builder, "wheel_emulation_vertical");
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(object), vertical_enable);
+
     g_free(values);
 }
 
@@ -473,22 +532,17 @@ setup_current_values (GpdsUI *ui, GtkBuilder *builder)
                                          builder,
                                          "wheel_emulation_inertia");
 
-    set_scroll_property_from_preference(track_point_ui,
-                                        GPDS_TRACK_POINT_WHEEL_EMULATION_Y_AXIS,
-                                        GPDS_TRACK_POINT_WHEEL_EMULATION_Y_AXIS_KEY,
-                                        builder,
-                                        "wheel_emulation_vertical");
-    set_scroll_property_from_preference(track_point_ui,
-                                        GPDS_TRACK_POINT_WHEEL_EMULATION_X_AXIS,
-                                        GPDS_TRACK_POINT_WHEEL_EMULATION_X_AXIS_KEY,
-                                        builder,
-                                        "wheel_emulation_horizontal");
+    set_scroll_axes_property_from_preference(track_point_ui,
+                                             builder);
 }
 
 static gboolean
 is_available (GpdsUI *ui, GError **error)
 {
-    if (!gpds_xinput_exist_device(GPDS_TRACK_POINT_DEVICE_NAME)) {
+    const gchar *device_name;
+    device_name = gpds_track_point_xinput_find_device_name();
+
+    if (!device_name) {
         g_set_error(error,
                     GPDS_XINPUT_ERROR,
                     GPDS_XINPUT_ERROR_NO_DEVICE,
@@ -506,6 +560,8 @@ is_available (GpdsUI *ui, GError **error)
         return FALSE;
     }
 
+    GPDS_TRACK_POINT_UI(ui)->device_name = g_strdup(device_name);
+
     return TRUE;
 }
 
@@ -522,7 +578,7 @@ build (GpdsUI  *ui, GError **error)
         return FALSE;
     }
 
-    GPDS_TRACK_POINT_UI(ui)->xinput = gpds_xinput_new(GPDS_TRACK_POINT_DEVICE_NAME);
+    GPDS_TRACK_POINT_UI(ui)->xinput = gpds_xinput_new(GPDS_TRACK_POINT_UI(ui)->device_name);
 
     setup_current_values(ui, builder);
     setup_signals(ui, builder);
